@@ -40,7 +40,8 @@ _defaults_file = os.path.join(os.path.dirname(__file__), 'defaults.jhbuildrc')
 _default_jhbuildrc = os.path.join(os.environ['HOME'], '.jhbuildrc')
 
 _known_keys = [ 'moduleset', 'modules', 'skip', 'tags', 'prefix',
-                'checkoutroot', 'buildroot', 'autogenargs', 'makeargs',
+                'partial_build', 'checkoutroot', 'buildroot', 'top_builddir',
+                'autogenargs', 'makeargs', 'nice_build',
                 'installprog', 'repos', 'branches', 'noxvfb', 'xvfbargs',
                 'builddir_pattern', 'module_autogenargs', 'module_makeargs',
                 'interact', 'buildscript', 'nonetwork',
@@ -58,7 +59,8 @@ _known_keys = [ 'moduleset', 'modules', 'skip', 'tags', 'prefix',
                 'jhbuildbot_mastercfg', 'use_local_modulesets',
                 'ignore_suggests', 'modulesets_dir', 'mirror_policy',
                 'module_mirror_policy', 'dvcs_mirror_dir', 'build_targets',
-                'cmakeargs', 'module_cmakeargs' ]
+                'cmakeargs', 'module_cmakeargs', 'print_command_pattern',
+                'static_analyzer', 'module_static_analyzer', 'static_analyzer_template', 'static_analyzer_outputdir' ]
 
 env_prepends = {}
 def prependpath(envvar, path):
@@ -108,7 +110,8 @@ def addpath(envvar, path):
             if sys.platform.startswith('win'):
                 path = jhbuild.utils.subprocess_win32.fix_path_for_msys(path)
 
-            if sys.platform.startswith('win') and path[1]==':':
+            if sys.platform.startswith('win') and len(path) > 1 and \
+               path[1] == ':':
                 # Windows: Don't allow c:/ style paths in :-separated env vars
                 # for obvious reasons. /c/ style paths are valid - if a var is
                 # separated by : it will only be of interest to programs inside
@@ -265,6 +268,10 @@ class Config:
         # default tarballdir to checkoutroot
         if not self.tarballdir: self.tarballdir = self.checkoutroot
 
+        # Ensure top_builddir is absolute
+        if not os.path.isabs(self.top_builddir):
+            self.top_builddir = os.path.join(self.prefix, self.top_builddir)
+
         # check possible checkout_mode values
         seen_copy_mode = (self.checkout_mode == 'copy')
         possible_checkout_modes = ('update', 'clobber', 'export', 'copy')
@@ -285,6 +292,9 @@ class Config:
                 self.use_local_modulesets = False
             self.modulesets_dir = None
 
+    def get_original_environment(self):
+        return self._orig_environ
+
     def setup_env(self):
         '''set environment variables for using prefix'''
 
@@ -294,8 +304,22 @@ class Config:
             except:
                 raise FatalError(_('install prefix (%s) can not be created') % self.prefix)
 
+        if not os.path.exists(self.top_builddir):
+            try:
+                os.makedirs(self.top_builddir)
+            except OSError:
+                raise FatalError(
+                        _('working directory (%s) can not be created') % self.top_builddir)
+
+        os.environ['JHBUILD_PREFIX'] = self.prefix
+
         os.environ['UNMANGLED_LD_LIBRARY_PATH'] = os.environ.get('LD_LIBRARY_PATH', '')
 
+        if not os.environ.get('DBUS_SYSTEM_BUS_ADDRESS'):
+            # Use the distribution's D-Bus for the system bus. JHBuild's D-Bus
+            # will # be used for the session bus
+            os.environ['DBUS_SYSTEM_BUS_ADDRESS'] = 'unix:path=/var/run/dbus/system_bus_socket'
+
         # LD_LIBRARY_PATH
         if self.use_lib64:
             libdir = os.path.join(self.prefix, 'lib64')
@@ -330,13 +354,12 @@ class Config:
         addpath('MANPATH', '')
         addpath('MANPATH', manpathdir)
 
+        # INFOPATH
+        infopathdir = os.path.join(self.prefix, 'share', 'info')
+        addpath('INFOPATH', infopathdir)
+
         # PKG_CONFIG_PATH
-        if os.environ.get('PKG_CONFIG_PATH') is None:
-            # add system pkgconfig lookup-directories by default, as pkg-config
-            # usage spread and is now used by libraries that are out of jhbuild
-            # realm; this also helps when building a single module with
-            # jhbuild.  It is possible to avoid this by setting PKG_CONFIG_PATH
-            # to the empty string.
+        if os.environ.get('PKG_CONFIG_PATH') is None and self.partial_build:
             for dirname in ('share', 'lib', 'lib64'):
                 full_name = '/usr/%s/pkgconfig' % dirname
                 if os.path.exists(full_name):
@@ -358,10 +381,14 @@ class Config:
         addpath('GI_TYPELIB_PATH', typelibpath)
 
         # XDG_DATA_DIRS
+        if self.partial_build:
+            addpath('XDG_DATA_DIRS', '/usr/share')
         xdgdatadir = os.path.join(self.prefix, 'share')
         addpath('XDG_DATA_DIRS', xdgdatadir)
 
         # XDG_CONFIG_DIRS
+        if self.partial_build:
+            addpath('XDG_CONFIG_DIRS', '/etc')
         xdgconfigdir = os.path.join(self.prefix, 'etc', 'xdg')
         addpath('XDG_CONFIG_DIRS', xdgconfigdir)
 
@@ -376,10 +403,11 @@ class Config:
                 os.makedirs(aclocaldir)
             except:
                 raise FatalError(_("Can't create %s directory") % aclocaldir)
-        if os.path.exists('/usr/share/aclocal'):
-            addpath('ACLOCAL_FLAGS', '/usr/share/aclocal')
-        if os.path.exists('/usr/local/share/aclocal'):
-            addpath('ACLOCAL_FLAGS', '/usr/local/share/aclocal')
+        if self.partial_build:
+            if os.path.exists('/usr/share/aclocal'):
+                addpath('ACLOCAL_FLAGS', '/usr/share/aclocal')
+                if os.path.exists('/usr/local/share/aclocal'):
+                    addpath('ACLOCAL_FLAGS', '/usr/local/share/aclocal')
         addpath('ACLOCAL_FLAGS', aclocaldir)
 
         # PERL5LIB
@@ -389,7 +417,7 @@ class Config:
         # These two variables are so that people who use "jhbuild shell"
         # can tweak their shell prompts and such to show "I'm under jhbuild".
         # The first variable is the obvious one to look for; the second
-        # one is for historical reasons.
+        # one is for historical reasons. 
         os.environ['UNDER_JHBUILD'] = 'true'
         os.environ['CERTIFIED_GNOMIE'] = 'yes'
 
@@ -554,6 +582,7 @@ class Config:
         if k == 'quiet_mode' and v:
             try:
                 import curses
+                logging.getLogger().setLevel(logging.ERROR)
             except ImportError:
                 logging.warning(
                         _('quiet mode has been disabled because the Python curses module is missing.'))
